import hashlib
import base64
import socket
import struct
import time
from uuid import uuid4
from fastapi import HTTPException, Request

from xcap.configuration import AuthenticationConfig

# In-memory user database (username: password)
users = {
    "user1": "password123",
    "user2": "secret456",
    "admin": "adminpassword",
}

# In-memory nonce cache with expiration time (for demonstration purposes)
nonce_cache = {}
NONCE_EXPIRATION_TIME = 300  # 5 minutes for nonce expiration


class AuthenticationManager:
    def __init__(self):
        self.nonce_cache = nonce_cache
        self.users = users
        self.trusted_peers = AuthenticationConfig.trusted_peers

    # Helper function to generate a nonce
    def generate_nonce(self) -> str:
        """Generate a new nonce (typically a random string with a timestamp)."""
        timestamp = int(time.time())
        unique_nonce = f"{timestamp}-{uuid4()}"
        return base64.b64encode(unique_nonce.encode()).decode("utf-8")

    # Helper function to create the Digest response hash
    def create_digest_response(self, username: str, password: str, nonce: str, uri: str, method: str) -> str:
        """Create the digest response using MD5 hashes."""
        ha1 = hashlib.md5(f"{username}:XCAP Server:{password}".encode()).hexdigest()
        ha2 = hashlib.md5(f"{method}:{uri}".encode()).hexdigest()
        return hashlib.md5(f"{ha1}:{nonce}:{ha2}".encode()).hexdigest()

    # Function to validate and clean up expired nonces
    def validate_nonce(self, nonce: str) -> bool:
        """Check if the nonce is valid and not expired."""
        if nonce not in self.nonce_cache:
            return False
        timestamp, _ = self.nonce_cache[nonce]
        current_time = int(time.time())
        if current_time - timestamp > NONCE_EXPIRATION_TIME:
            # Nonce expired, remove it from the cache
            del self.nonce_cache[nonce]
            return False
        return True

    # Function to check if the client IP is in the trusted peers list
    def is_ip_trusted(self, client_ip: str) -> bool:
        """Check if the client IP is in the trusted peers list."""
        if not self.trusted_peers:
            return False

        # Iterate through each network range in the trusted_parties list
        for range in self.trusted_peers:
            # Convert the IP address to a 32-bit integer
            ip_int = struct.unpack('!L', socket.inet_aton(client_ip.decode()))[0]

            # Perform the bitwise comparison (IP address & network mask == base address)
            if ip_int & range[1] == range[0]:
                return True

        # If the IP address does not match any range, return False
        return False

    # Digest Authentication Dependency
    def digest_auth(self, request: Request):
        auth_header = request.headers.get("Authorization")
        if not auth_header or not auth_header.startswith("Digest "):
            raise HTTPException(status_code=401, detail="Digest authentication required")

        # Parse the Digest fields from the header
        try:
            auth_fields = {k: v.strip('"') for k, v in (field.split("=") for field in auth_header[7:].split(", "))}
        except ValueError:
            raise HTTPException(status_code=401, detail="Invalid Digest authentication format")

        required_fields = ["username", "nonce", "response", "uri"]
        for field in required_fields:
            if field not in auth_fields:
                raise HTTPException(status_code=401, detail=f"Missing required Digest field: {field}")

        username = auth_fields["username"]
        nonce = auth_fields["nonce"]
        response = auth_fields["response"]
        uri = request.url.path
        method = request.method

        if not self.validate_nonce(nonce):
            raise HTTPException(status_code=401, detail="Invalid or expired nonce")

        if username not in self.users or response != self.create_digest_response(username, self.users[username], nonce, uri, method):
            raise HTTPException(status_code=401, detail="Invalid credentials or response")

        return username

    # Basic Authentication Dependency
    def basic_auth(self, request: Request):
        auth_header = request.headers.get("Authorization")
        if not auth_header or not auth_header.startswith("Basic "):
            raise HTTPException(status_code=401, detail="Basic authentication required")

        auth_value = auth_header[6:].strip()
        decoded_value = base64.b64decode(auth_value).decode("utf-8")
        username, password = decoded_value.split(":")

        if username not in self.users or self.users[username] != password:
            raise HTTPException(status_code=401, detail="Invalid credentials")

        return username

    # Function to handle IP-based authentication and fallbacks to Digest or Basic
    async def authenticate_request(self, request: Request):
        """Authenticate a request by checking IP and applying Digest or Basic authentication as needed."""
        client_ip = request.client.host

        if self.is_ip_trusted(client_ip):
            return None  # Skip authentication for trusted IPs

        username = None
        try:
            username = await self.digest_auth(request)
        except HTTPException:
            username = await self.basic_auth(request)

        return username

