from sqlmodel import Session, select
from xcap.db.models import XCAPResource
from . import StorageInterface
from typing import Optional

class DatabaseStorage(StorageInterface):
    def get_data(self, db: Session, domain: str, namespace: str, username: str, resource_id: str) -> Optional[dict]:
        """Retrieve XCAP data from the database."""
        resource = db.execute(select(XCAPResource).filter(
            XCAPResource.resource_id == resource_id,
            XCAPResource.username == username,
            XCAPResource.namespace == namespace,
            XCAPResource.domain == domain
        )).scalar_one_or_none()
        if resource:
            return {
                "resource_id": resource.resource_id,
                "name": resource.name,
                "content": resource.content
            }
        return None

    def save_data(self, db: Session, domain: str, namespace: str, username: str, resource_id: str, data: dict) -> dict:
        """Save XCAP data into the database."""
        resource = XCAPResource(
            resource_id=resource_id,
            name=data['name'],
            content=data['content'],
            domain=domain,
            namespace=namespace,
            username=username
        )
        db.add(resource)
        db.commit()
        db.refresh(resource)
        return {
            "resource_id": resource.resource_id,
            "name": resource.name,
            "content": resource.content
        }

    def update_data(self, db: Session, domain: str, namespace: str, username: str, resource_id: str, data: dict) -> Optional[dict]:
        """Update existing XCAP data in the database."""
        resource = db.execute(select(XCAPResource).filter(
            XCAPResource.resource_id == resource_id,
            XCAPResource.username == username,
            XCAPResource.namespace == namespace,
            XCAPResource.domain == domain
        )).scalar_one_or_none()
        if resource:
            resource.name = data['name']
            resource.content = data['content']
            db.commit()
            db.refresh(resource)
            return {
                "resource_id": resource.resource_id,
                "name": resource.name,
                "content": resource.content
            }
        return None

    def delete_data(self, db: Session, domain: str, namespace: str, username: str, resource_id: str) -> bool:
        """Delete XCAP data from the database."""
        resource = db.execute(select(XCAPResource).filter(
            XCAPResource.resource_id == resource_id,
            XCAPResource.username == username,
            XCAPResource.namespace == namespace,
            XCAPResource.domain == domain
        )).scalar_one_or_none()
        if resource:
            db.delete(resource)
            db.commit()
            return True
        return False

