from fastapi import APIRouter, HTTPException, Depends, Request
from xcap.authentication import AuthenticationManager
from xcap.schemas import XCAPData

from xcap.services.xcap_service import get_xcap_data, save_xcap_data, update_xcap_data, delete_xcap_data

# Create an instance of the APIRouter class
router = APIRouter()

auth_manager = AuthenticationManager()

# Default domain if not provpathed
DEFAULT_DOMAIN = "default.com"

# Define a route to retrieve XCAP data by domain, namespace, username, and resource_path
@router.get("/xcap-root/{namespace}/{resource_path:path}", response_model=XCAPData)
@router.get("/xcap-root@{domain}/{namespace}/{resource_path:path}", response_model=XCAPData)
async def read_xcap_data(
    namespace: str,
    resource_path: str,
    request: Request,
    username: str = Depends(auth_manager.authenticate_request),
    domain: str = DEFAULT_DOMAIN  # default to "default.com" if not provpathed in the URL,
):
    """Retrieve XCAP data based on domain, namespace, username, and resource path."""
    xcap_data = await get_xcap_data(domain, namespace, username, resource_path)
    if not xcap_data:
        raise HTTPException(status_code=404, detail="Resource not found")
    return xcap_data


# Define a route to create new XCAP data for a domain
@router.post("/xcap-root/{namespace}/{resource_path:path}")
@router.post("/xcap-root@{domain}/{namespace}/{resource_path:path}")
async def post_not_allowed(request: Request):
    raise HTTPException(status_code=405, detail="POST method is not allowed here")

# Define a route to update XCAP data for a domain
@router.put("/xcap-root/{namespace}/{resource_path:path}", response_model=XCAPData)
@router.put("/xcap-root@{domain}/{namespace}/{resource_path:path}", response_model=XCAPData)
async def update_xcap_data_route(
    namespace: str,
    resource_path: str,
    xcap_data: XCAPData,
    request: Request,
    username: str = Depends(auth_manager.authenticate_request),
    domain: str = DEFAULT_DOMAIN,  # default to "default.com" if not provpathed in the URL
):
    """Update an existing XCAP resource for the specified domain, namespace, username, and resource path."""
    updated_data = await update_xcap_data(domain, namespace, username, resource_path, xcap_data)
    if not updated_data:
        raise HTTPException(status_code=404, detail="Resource not found")
    return updated_data

# Define a route to delete XCAP data for a domain
@router.delete("/xcap-root/{namespace}/{resource_path:path}", status_code=204)
@router.delete("/xcap-root@{domain}/{namespace}/{resource_path:path}", status_code=204)
async def delete_xcap_data_route(
    namespace: str,
    resource_path: str,
    request: Request,
    username: str = Depends(auth_manager.authenticate_request),
    domain: str = DEFAULT_DOMAIN  # default to "default.com" if not provpathed in the URL
):
    """Delete an XCAP resource for the specified domain, namespace, username, and resource path."""
    success = await delete_xcap_data(domain, namespace, username, resource_path)
    if not success:
        raise HTTPException(status_code=404, detail="Resource not found")
    return {"message": "Resource deleted successfully"}

